@@ -1,12 +1,12 @@
 import os
 from pathlib import Path
 
+from jupyter_releaser.actions.common import run_action
 from jupyter_releaser.changelog import get_version_entry
-from jupyter_releaser.util import CHECKOUT_NAME
-from jupyter_releaser.util import run
+from jupyter_releaser.util import CHECKOUT_NAME, get_branch
 
 target = os.environ.get("RH_REPOSITORY")
-branch = os.environ.get("RH_BRANCH")
+branch = os.environ.get("RH_BRANCH", "<default>")
 ref = os.environ.get("RH_REF")
 since = os.environ.get("RH_SINCE")
 until = os.environ.get("RH_UNTIL")
@@ -17,7 +17,8 @@ print("target:", target)
 print("branch:", branch)
 print("convert to rst:", convert_to_rst)
 
-run("jupyter-releaser prep-git")
+run_action("jupyter-releaser prep-git")
+branch = get_branch()
 orig_dir = os.getcwd()
 os.chdir(CHECKOUT_NAME)
 output = get_version_entry(ref, branch, target, "current", since=since, until=until)
