@@ -3,7 +3,6 @@
 import re
 from pathlib import Path
 
-from ghapi.core import GhApi
 from github_activity import generate_activity_md
 
 from jupyter_releaser import util
@@ -11,9 +10,10 @@ from jupyter_releaser import util
 START_MARKER = "<!-- <START NEW CHANGELOG ENTRY> -->"
 END_MARKER = "<!-- <END NEW CHANGELOG ENTRY> -->"
 PR_PREFIX = "Automated Changelog Entry"
+PRECOMMIT_PREFIX = "[pre-commit.ci] pre-commit autoupdate"
 
 
-def format_pr_entry(target, number, auth=None):
+def format_pr_entry(target, number, auth=None, dry_run=False):
     """Format a PR entry in the style used by our changelogs.
 
     Parameters
@@ -24,6 +24,8 @@ def format_pr_entry(target, number, auth=None):
         The PR number to resolve
     auth : str, optional
         The GitHub authorization token
+    dry_run: bool, optional
+        Whether this is a dry run.
 
     Returns
     -------
@@ -31,7 +33,7 @@ def format_pr_entry(target, number, auth=None):
         A formatted PR entry
     """
     owner, repo = target.split("/")
-    gh = GhApi(owner=owner, repo=repo, token=auth)
+    gh = util.get_gh_object(dry_run=dry_run, owner=owner, repo=repo, token=auth)
     pull = gh.pulls.get(number)
     title = pull.title
     url = pull.html_url
@@ -51,6 +53,7 @@ def get_version_entry(
     until=None,
     auth=None,
     resolve_backports=False,
+    dry_run=False,
 ):
     """Get a changelog for the changes since the last tag on the given branch.
 
@@ -74,13 +77,19 @@ def get_version_entry(
         The GitHub authorization token
     resolve_backports: bool, optional
         Whether to resolve backports to the original PR
+    dry_run: bool, optional
+        Whether this is a dry run.
 
     Returns
     -------
     str
         A formatted changelog entry with markers
     """
-    since = since or _get_since(ref or branch, since_last_stable)
+    branch = branch or util.get_branch()
+    since = since or util.get_latest_tag(ref or branch, since_last_stable)
+
+    if since == "":
+        since = util.get_first_commit(ref or branch)
 
     util.log(f"Getting changes to {repo} since {since} on branch {branch}...")
 
@@ -104,15 +113,19 @@ def get_version_entry(
         return f"## {version}\n\nNo merged PRs"
 
     entry = md.replace("[full changelog]", "[Full Changelog]")
-    entry = entry.replace("...None", f"...{until}")
+
+    if until:
+        entry = entry.replace("...None", f"...{until}")
+    else:
+        entry = entry.replace("...None", "")
 
     entry = entry.splitlines()[2:]
 
     for (ind, line) in enumerate(entry):
-        if re.search(r"\[@meeseeksmachine\]", line) is not None:
-            match = re.search(r"Backport PR #(\d+)", line)
-            if match:
-                entry[ind] = format_pr_entry(repo, match.groups()[0])
+        # Look for a backport, either manual or automatic.
+        match = re.search(r"Backport PR #(\d+) on branch", line)
+        if match:
+            entry[ind] = format_pr_entry(repo, match.groups()[0], dry_run=dry_run)
 
     # Remove github actions PRs
     gh_actions = "[@github-actions](https://github.com/github-actions)"
@@ -121,13 +134,14 @@ def get_version_entry(
     # Remove automated changelog PRs
     entry = [e for e in entry if PR_PREFIX not in e]
 
+    # Remove Pre-Commit PRs
+    entry = [e for e in entry if PRECOMMIT_PREFIX not in e]
+
     entry = "\n".join(entry).strip()
 
-    # Replace "*" unordered list marker with "-" since this is what
-    # Prettier uses
-    # TODO: remove after github_activity 0.1.4+ is available
-    entry = re.sub(r"^\* ", "- ", entry)
-    entry = re.sub(r"\n\* ", "\n- ", entry)
+    # Remove empty documentation entry if only automated changelogs were there
+    if "# Documentation improvements" in entry and "# Documentation improvements\n\n-" not in entry:
+        entry = re.sub(r"#+ Documentation improvements\n\n", "", entry)
 
     output = f"""
 ## {version}
@@ -148,15 +162,6 @@ def build_entry(
     # Get the new version
     version = util.get_version()
 
-    # Get the existing changelog and run some validation
-    changelog = Path(changelog_path).read_text(encoding="utf-8")
-
-    if START_MARKER not in changelog or END_MARKER not in changelog:
-        raise ValueError("Missing insert marker for changelog")
-
-    if changelog.find(START_MARKER) != changelog.rfind(START_MARKER):
-        raise ValueError("Insert marker appears more than once in changelog")
-
     # Get changelog entry
     entry = get_version_entry(
         ref,
@@ -168,13 +173,25 @@ def build_entry(
         auth=auth,
         resolve_backports=resolve_backports,
     )
+    update_changelog(changelog_path, entry)
+
+
+def update_changelog(changelog_path, entry):
+    # Get the new version
+    version = util.get_version()
+
+    # Get the existing changelog and run some validation
+    changelog = Path(changelog_path).read_text(encoding="utf-8")
+
+    if START_MARKER not in changelog or END_MARKER not in changelog:
+        raise ValueError("Missing insert marker for changelog")
+
+    if changelog.find(START_MARKER) != changelog.rfind(START_MARKER):
+        raise ValueError("Insert marker appears more than once in changelog")
 
     changelog = insert_entry(changelog, entry, version=version)
     Path(changelog_path).write_text(changelog, encoding="utf-8")
 
-    # Stage changelog
-    util.run(f"git add {util.normalize_path(changelog_path)}")
-
 
 def insert_entry(changelog, entry, version=None):
     """Insert the entry into the existing changelog."""
@@ -192,7 +209,7 @@ def insert_entry(changelog, entry, version=None):
             pr = re.search(r"\[#\d+\]", line)
             if not pr:
                 continue
-            for old_line in prev_entry.splitlines():
+            for old_line in old_lines:
                 if pr.group() in old_line:
                     lines[ind] = old_line
         changelog = changelog.replace(prev_entry, "\n".join(lines))
@@ -269,16 +286,50 @@ def check_entry(
                 break
         if skip:
             continue
-        if not f"[#{pr}]" in final_entry:  # pragma: no cover
+        if f"[#{pr}]" not in final_entry:  # pragma: no cover
             raise ValueError(f"Missing PR #{pr} in changelog")
     for pr in final_prs:
-        if not f"[#{pr}]" in raw_entry:  # pragma: no cover
+        if f"[#{pr}]" not in raw_entry:  # pragma: no cover
             raise ValueError(f"PR #{pr} does not belong in changelog for {version}")
 
     if output:
         Path(output).write_text(final_entry, encoding="utf-8")
 
 
+def splice_github_entry(orig_entry, github_entry):
+    """Splice an entry created on GitHub into one created by build_entry"""
+
+    # Override PR titles
+    gh_regex = re.compile(r"^\* (.*?) by @.*?/pull/(\d+)$", flags=re.MULTILINE)
+    cl_regex = re.compile(r"^- (.*?) \[#(\d+)\]")
+
+    lut = {}
+    for title, pr in re.findall(gh_regex, github_entry):
+        lut[pr] = title
+
+    lines = orig_entry.splitlines()
+    for (ind, line) in enumerate(lines):
+        match = re.match(cl_regex, line)
+        if not match:
+            continue
+        title, pr = re.findall(cl_regex, line)[0]
+        if pr in lut:
+            lines[ind] = line.replace(title, lut[pr])
+
+    # Handle preamble
+    preamble_index = github_entry.index("## What's Changed")
+    if preamble_index > 0:
+        preamble = github_entry[:preamble_index]
+        if preamble.startswith("# "):
+            preamble = preamble.replace("# ", "## ")
+        if preamble.startswith("## "):
+            preamble = preamble.replace("## ", "### ")
+
+        lines = preamble.splitlines() + [""] + lines
+
+    return "\n".join(lines)
+
+
 def extract_current(changelog_path):
     """Extract the current changelog entry"""
     body = ""
@@ -292,21 +343,10 @@ def extract_current(changelog_path):
     return body
 
 
-def _get_since(source, since_last_stable=False):
-    """Get the appropriate since reference or None"""
-    tags = util.run(
-        f"git --no-pager tag --sort=-creatordate --merged {source}", quiet=True
-    )
-    if not tags:
-        return
-
-    tags = tags.splitlines()
-
-    if since_last_stable:
-        stable_tag = re.compile(r"\d\.\d\.\d$")
-        tags = [t for t in tags if re.search(stable_tag, t)]
-        if not tags:
-            return
-        return tags[0]
-
-    return tags[0]
+def extract_current_version(changelog_path):
+    """Extract the current released version from the changelog"""
+    body = extract_current(changelog_path)
+    match = re.match(r"#+ (\d\S+)", body.strip())
+    if not match:
+        raise ValueError("Could not find previous version")
+    return match.groups()[0]
