@@ -2,37 +2,34 @@
 # Distributed under the terms of the Modified BSD License.
 import os
 import os.path as osp
+import typing as t
 from glob import glob
 from pathlib import Path
 
 import click
 
-from jupyter_releaser import changelog
-from jupyter_releaser import lib
-from jupyter_releaser import npm
-from jupyter_releaser import python
-from jupyter_releaser import util
+from jupyter_releaser import changelog, lib, npm, python, util
 
 
 class ReleaseHelperGroup(click.Group):
     """Click group tailored to jupyter-releaser"""
 
-    _needs_checkout_dir = dict()
+    _needs_checkout_dir: t.Dict[str, bool] = {}
 
     def invoke(self, ctx):
         """Handle jupyter-releaser config while invoking a command"""
         # Get the command name and make sure it is valid
         cmd_name = ctx.protected_args[0]
-        if not cmd_name in self.commands:
+        if cmd_name not in self.commands:
             super().invoke(ctx)
 
         if cmd_name == "list-envvars":
-            envvars = dict()
+            envvars: t.Dict[str, str] = {}
             for cmd_name in self.commands:
                 for param in self.commands[cmd_name].params:
                     if isinstance(param, click.Option):
                         if param.envvar:
-                            envvars[param.name] = param.envvar
+                            envvars[str(param.name)] = str(param.envvar)
 
             for key in sorted(envvars):
                 util.log(f"{key.replace('_', '-')}: {envvars[key]}")
@@ -51,30 +48,45 @@ class ReleaseHelperGroup(click.Group):
         hooks = config.get("hooks", {})
         options = config.get("options", {})
         skip = config.get("skip", [])
+
         if "--force" in ctx.args:
             skip = []
             ctx.args.remove("--force")
 
+        skip += os.environ.get("RH_STEPS_TO_SKIP", "").split(",")
+
         # Print a separation header
         util.log(f'\n\n{"-" * 50}')
-        util.log(cmd_name)
-        util.log(f'{"-" * 50}\n\n')
+        util.log(f"\n\n{cmd_name}\n\n")
+        util.log(f'\n\n{"-" * 50}')
 
         if cmd_name in skip or cmd_name.replace("-", "_") in skip:
             util.log("*** Skipping based on skip config")
+            util.log(f'{"-" * 50}\n\n')
             return
 
         # Handle all of the parameters
         for param in self.commands[cmd_name].get_params(ctx):
-            # Defer to env var overrides
-            if param.envvar and os.environ.get(param.envvar):
-                continue
             name = param.name
+            assert name is not None
+
+            # Defer to env var overrides
+            if param.envvar and os.environ.get(str(param.envvar)):
+                value = os.environ[str(param.envvar)]
+                if "token" in name.lower():
+                    value = "***"
+                util.log(f"Using env value for {name}: '{value}'")
+                continue
+
+            # Handle cli and options overrides.
             if name in options or name.replace("_", "-") in options:
                 arg = f"--{name.replace('_', '-')}"
                 # Defer to cli overrides
                 if arg not in ctx.args:
                     val = options.get(name, options.get(name.replace("_", "-")))
+                    if "token" in arg.lower():
+                        val = "***"
+                    util.log(f"Adding option override for {arg}: '{val}")
                     if isinstance(val, list):
                         for v in val:
                             ctx.args.append(arg)
@@ -82,6 +94,17 @@ class ReleaseHelperGroup(click.Group):
                     else:
                         ctx.args.append(arg)
                         ctx.args.append(val)
+                    continue
+                else:
+                    val = ctx.args[ctx.args.index(arg) + 1]
+                    if "token" in name.lower():
+                        val = "***"
+                    util.log(f"Using cli arg for {name}: '{val}'")
+                    continue
+
+            util.log(f"Using default value for {name}: '{param.default}'")
+
+        util.log(f'{"~" * 50}\n\n')
 
         # Handle before hooks
         before = f"before-{cmd_name}"
@@ -123,9 +146,7 @@ class ReleaseHelperGroup(click.Group):
 
 
 @click.group(cls=ReleaseHelperGroup)
-@click.option(
-    "--force", default=False, help="Force a command to run even when skipped by config"
-)
+@click.option("--force", default=False, help="Force a command to run even when skipped by config")
 def main(force):
     """Jupyter Releaser scripts"""
     pass
@@ -141,10 +162,24 @@ version_spec_options = [
     )
 ]
 
-version_cmd_options = [
+
+post_version_spec_options = [
     click.option(
-        "--version-cmd", envvar="RH_VERSION_COMMAND", help="The version command"
-    )
+        "--post-version-spec",
+        envvar="RH_POST_VERSION_SPEC",
+        default="",
+        help="The post release version (usually dev)",
+    ),
+    click.option(
+        "--post-version-message",
+        default="Bumped version to {post_version}",
+        envvar="RH_POST_VERSION_MESSAGE",
+        help="The post release message",
+    ),
+]
+
+version_cmd_options = [
+    click.option("--version-cmd", envvar="RH_VERSION_COMMAND", help="The version command")
 ]
 
 
@@ -158,9 +193,7 @@ auth_options = [
     click.option("--auth", envvar="GITHUB_ACCESS_TOKEN", help="The GitHub auth token"),
 ]
 
-username_options = [
-    click.option("--username", envvar="GITHUB_ACTOR", help="The git username")
-]
+username_options = [click.option("--username", envvar="GITHUB_ACTOR", help="The git username")]
 
 dist_dir_options = [
     click.option(
@@ -171,15 +204,36 @@ dist_dir_options = [
     )
 ]
 
-dry_run_options = [
+python_packages_options = [
     click.option(
-        "--dry-run", is_flag=True, envvar="RH_DRY_RUN", help="Run as a dry run"
+        "--python-packages",
+        envvar="RH_PYTHON_PACKAGES",
+        default=["."],
+        multiple=True,
+        help='The list of strings of the form "path_to_package:name_of_package"',
     )
 ]
 
+check_imports_options = [
+    click.option(
+        "--check-imports",
+        envvar="RH_CHECK_IMPORTS",
+        default=[],
+        multiple=True,
+        help="The Python packages import to check for; default to the Python package name.",
+    )
+]
 
-git_url_options = [
-    click.option("--git-url", help="A custom url for the git repository")
+dry_run_options = [
+    click.option("--dry-run", is_flag=True, envvar="RH_DRY_RUN", help="Run as a dry run")
+]
+
+
+git_url_options = [click.option("--git-url", help="A custom url for the git repository")]
+
+
+release_url_options = [
+    click.option("--release-url", envvar="RH_RELEASE_URL", help="A draft GitHub release url")
 ]
 
 
@@ -231,6 +285,21 @@ npm_install_options = [
     )
 ]
 
+pydist_check_options = [
+    click.option(
+        "--pydist-check-cmd",
+        envvar="RH_PYDIST_CHECK_CMD",
+        default="pipx run twine check --strict",
+        help="The command to use to check a python distribution file",
+    ),
+    click.option(
+        "--pydist-resource-paths",
+        envvar="RH_PYDIST_RESOURCE_PATHS",
+        multiple=True,
+        help="Resource paths that should be available when installed",
+    ),
+]
+
 
 def add_options(options):
     """Add extracted common options to a click command"""
@@ -273,10 +342,26 @@ def prep_git(ref, branch, repo, auth, username, git_url):
 @main.command()
 @add_options(version_spec_options)
 @add_options(version_cmd_options)
+@add_options(changelog_path_options)
+@add_options(python_packages_options)
 @use_checkout_dir()
-def bump_version(version_spec, version_cmd):
+def bump_version(version_spec, version_cmd, changelog_path, python_packages):
     """Prep git and env variables and bump version"""
-    lib.bump_version(version_spec, version_cmd)
+    prev_dir = os.getcwd()
+    for python_package in [p.split(":")[0] for p in python_packages]:
+        os.chdir(python_package)
+        lib.bump_version(version_spec, version_cmd, changelog_path)
+        os.chdir(prev_dir)
+
+
+@main.command()
+@add_options(dry_run_options)
+@add_options(auth_options)
+@add_options(changelog_path_options)
+@add_options(release_url_options)
+@use_checkout_dir()
+def extract_changelog(dry_run, auth, changelog_path, release_url):
+    lib.extract_changelog(dry_run, auth, changelog_path, release_url)
 
 
 @main.command()
@@ -305,6 +390,7 @@ def build_changelog(
 @add_options(auth_options)
 @add_options(changelog_path_options)
 @add_options(dry_run_options)
+@add_options(post_version_spec_options)
 @use_checkout_dir()
 def draft_changelog(
     version_spec,
@@ -316,10 +402,13 @@ def draft_changelog(
     auth,
     changelog_path,
     dry_run,
+    post_version_spec,
+    post_version_message,
 ):
     """Create a changelog entry PR"""
     lib.draft_changelog(
         version_spec,
+        ref,
         branch,
         repo,
         since,
@@ -327,61 +416,49 @@ def draft_changelog(
         auth,
         changelog_path,
         dry_run,
-    )
-
-
-@main.command()
-@add_options(changelog_options)
-@click.option(
-    "--output", envvar="RH_CHANGELOG_OUTPUT", help="The output file for changelog entry"
-)
-@use_checkout_dir()
-def check_changelog(
-    ref,
-    branch,
-    repo,
-    auth,
-    changelog_path,
-    since,
-    since_last_stable,
-    resolve_backports,
-    output,
-):
-    """Check changelog entry"""
-    changelog.check_entry(
-        ref,
-        branch,
-        repo,
-        auth,
-        changelog_path,
-        since,
-        since_last_stable,
-        resolve_backports,
-        output,
+        post_version_spec,
+        post_version_message,
     )
 
 
 @main.command()
 @add_options(dist_dir_options)
+@add_options(python_packages_options)
 @use_checkout_dir()
-def build_python(dist_dir):
+def build_python(dist_dir, python_packages):
     """Build Python dist files"""
-    if not util.PYPROJECT.exists() and not util.SETUP_PY.exists():
-        util.log("Skipping build-python since there are no python package files")
-        return
-    python.build_dist(dist_dir)
+    prev_dir = os.getcwd()
+    clean = True
+    for python_package in [p.split(":")[0] for p in python_packages]:
+        os.chdir(python_package)
+        if not util.PYPROJECT.exists() and not util.SETUP_PY.exists():
+            util.log(
+                f"Skipping build-python in {python_package} since there are no python package files"
+            )
+        else:
+            python.build_dist(Path(os.path.relpath(".", python_package)) / dist_dir, clean=clean)
+            clean = False
+        os.chdir(prev_dir)
 
 
 @main.command()
 @add_options(dist_dir_options)
+@add_options(check_imports_options)
+@add_options(pydist_check_options)
 @use_checkout_dir()
-def check_python(dist_dir):
+def check_python(dist_dir, check_imports, pydist_check_cmd, pydist_resource_paths):
     """Check Python dist files"""
     for dist_file in glob(f"{dist_dir}/*"):
         if Path(dist_file).suffix not in [".gz", ".whl"]:
             util.log(f"Skipping non-python dist file {dist_file}")
             continue
-        python.check_dist(dist_file)
+
+        python.check_dist(
+            dist_file,
+            python_imports=check_imports,
+            check_cmd=pydist_check_cmd,
+            resource_paths=pydist_resource_paths,
+        )
 
 
 @main.command()
@@ -408,46 +485,6 @@ def check_npm(dist_dir, npm_install_options):
     npm.check_dist(dist_dir, npm_install_options)
 
 
-@main.command()
-@use_checkout_dir()
-def check_manifest():
-    """Check the project manifest"""
-    if util.PYPROJECT.exists() or util.SETUP_PY.exists():
-        util.run("check-manifest -v")
-    else:
-        util.log("Skipping check-manifest since there are no python package files")
-
-
-@main.command()
-@click.option(
-    "--ignore-glob",
-    default=[],
-    multiple=True,
-    help="Ignore test file paths based on glob pattern",
-)
-@click.option(
-    "--ignore-links",
-    multiple=True,
-    help="Ignore links based on regex pattern(s)",
-)
-@click.option(
-    "--cache-file",
-    envvar="RH_CACHE_FILE",
-    default="~/.cache/pytest-link-check",
-    help="The cache file to use",
-)
-@click.option(
-    "--links-expire",
-    default=604800,
-    envvar="RH_LINKS_EXPIRE",
-    help="Duration in seconds for links to be cached (default one week)",
-)
-@use_checkout_dir()
-def check_links(ignore_glob, ignore_links, cache_file, links_expire):
-    """Check URLs for HTML-containing files."""
-    lib.check_links(ignore_glob, ignore_links, cache_file, links_expire)
-
-
 @main.command()
 @add_options(dist_dir_options)
 @click.option(
@@ -474,58 +511,46 @@ def check_links(ignore_glob, ignore_links, cache_file, links_expire):
     help="Whether to skip tagging npm workspace packages",
 )
 @use_checkout_dir()
-def tag_release(
-    dist_dir, release_message, tag_format, tag_message, no_git_tag_workspace
-):
+def tag_release(dist_dir, release_message, tag_format, tag_message, no_git_tag_workspace):
     """Create release commit and tag"""
-    lib.tag_release(
-        dist_dir, release_message, tag_format, tag_message, no_git_tag_workspace
-    )
+    lib.tag_release(dist_dir, release_message, tag_format, tag_message, no_git_tag_workspace)
 
 
 @main.command()
 @add_options(branch_options)
+@add_options(version_cmd_options)
 @add_options(auth_options)
 @add_options(changelog_path_options)
-@add_options(version_cmd_options)
 @add_options(dist_dir_options)
 @add_options(dry_run_options)
-@click.option(
-    "--post-version-spec",
-    envvar="RH_POST_VERSION_SPEC",
-    help="The post release version (usually dev)",
-)
-@click.option(
-    "--post-version-message",
-    default="Bumped version to {post_version}",
-    envvar="RH_POST_VERSION_MESSAGE",
-    help="The post release message",
-)
+@add_options(release_url_options)
+@add_options(post_version_spec_options)
 @click.argument("assets", nargs=-1)
 @use_checkout_dir()
-def draft_release(
+def populate_release(
     ref,
     branch,
     repo,
+    version_cmd,
     auth,
     changelog_path,
-    version_cmd,
     dist_dir,
     dry_run,
+    release_url,
     post_version_spec,
     post_version_message,
     assets,
 ):
-    """Publish Draft GitHub release"""
-    lib.draft_release(
+    lib.populate_release(
         ref,
         branch,
         repo,
+        version_cmd,
         auth,
         changelog_path,
-        version_cmd,
         dist_dir,
         dry_run,
+        release_url,
         post_version_spec,
         post_version_message,
         assets,
@@ -534,22 +559,27 @@ def draft_release(
 
 @main.command()
 @add_options(auth_options)
-@click.argument("release-url", nargs=1)
+@add_options(dry_run_options)
+@add_options(release_url_options)
 @use_checkout_dir()
-def delete_release(auth, release_url):
+def delete_release(auth, dry_run, release_url):
     """Delete a draft GitHub release by url to the release page"""
-    lib.delete_release(auth, release_url)
+    lib.delete_release(auth, release_url, dry_run)
 
 
 @main.command()
 @add_options(auth_options)
 @add_options(dist_dir_options)
 @add_options(dry_run_options)
-@add_options(npm_install_options)
-@click.argument("release-url", nargs=1)
-def extract_release(auth, dist_dir, dry_run, release_url, npm_install_options):
+@add_options(release_url_options)
+def extract_release(auth, dist_dir, dry_run, release_url):
     """Download and verify assets from a draft GitHub release"""
-    lib.extract_release(auth, dist_dir, dry_run, release_url, npm_install_options)
+    lib.extract_release(
+        auth,
+        dist_dir,
+        dry_run,
+        release_url,
+    )
 
 
 @main.command()
@@ -565,7 +595,7 @@ def extract_release(auth, dist_dir, dry_run, release_url, npm_install_options):
     "--twine-cmd",
     help="The twine to run for Python release",
     envvar="TWINE_COMMAND",
-    default="twine upload",
+    default="pipx run twine upload",
 )
 @click.option(
     "--npm-registry",
@@ -574,29 +604,60 @@ def extract_release(auth, dist_dir, dry_run, release_url, npm_install_options):
     default="https://registry.npmjs.org/",
 )
 @click.option(
-    "--twine-registry",
-    help="The pypi register to target for publishing",
-    envvar="TWINE_REGISTRY",
-    default="https://pypi.org/simple/",
+    "--twine-repository-url",
+    help="The pypi registry to target for publishing",
+    envvar="TWINE_REPOSITORY_URL",
+    default="https://upload.pypi.org/legacy/",
 )
 @add_options(dry_run_options)
+@add_options(python_packages_options)
+@add_options(release_url_options)
 @use_checkout_dir()
 def publish_assets(
-    dist_dir, npm_token, npm_cmd, twine_cmd, npm_registry, twine_registry, dry_run
+    dist_dir,
+    npm_token,
+    npm_cmd,
+    twine_cmd,
+    npm_registry,
+    twine_repository_url,
+    dry_run,
+    release_url,
+    python_packages,
 ):
     """Publish release asset(s)"""
-    lib.publish_assets(
-        dist_dir, npm_token, npm_cmd, twine_cmd, npm_registry, twine_registry, dry_run
-    )
+    for python_package in python_packages:
+        lib.publish_assets(
+            dist_dir,
+            npm_token,
+            npm_cmd,
+            twine_cmd,
+            npm_registry,
+            twine_repository_url,
+            dry_run,
+            release_url,
+            python_package,
+        )
 
 
 @main.command()
 @add_options(auth_options)
-@click.argument("release-url", nargs=1)
+@add_options(dry_run_options)
+@add_options(release_url_options)
 @use_checkout_dir()
-def publish_release(auth, release_url):
+def publish_release(auth, dry_run, release_url):
     """Publish GitHub release"""
-    lib.publish_release(auth, release_url)
+    lib.publish_release(auth, dry_run, release_url)
+
+
+@main.command()
+@add_options(branch_options)
+@add_options(dry_run_options)
+@click.option(
+    "--expected-sha", help="The expected sha of the branch HEAD", envvar="RH_EXPECTED_SHA"
+)
+@use_checkout_dir()
+def ensure_sha(ref, branch, repo, dry_run, expected_sha):
+    util.ensure_sha(dry_run, expected_sha, branch)
 
 
 @main.command()
@@ -605,11 +666,9 @@ def publish_release(auth, release_url):
 @add_options(username_options)
 @add_options(changelog_path_options)
 @add_options(dry_run_options)
-@click.argument("release-url")
+@add_options(release_url_options)
 @use_checkout_dir()
-def forwardport_changelog(
-    auth, ref, branch, repo, username, changelog_path, dry_run, release_url
-):
+def forwardport_changelog(auth, ref, branch, repo, username, changelog_path, dry_run, release_url):
     """Forwardport Changelog Entries to the Default Branch"""
     lib.forwardport_changelog(
         auth, ref, branch, repo, username, changelog_path, dry_run, release_url
