@@ -1,28 +1,27 @@
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+import json
 import os
 import os.path as osp
 import re
 import shutil
+import tempfile
 import uuid
 from datetime import datetime
 from glob import glob
 from pathlib import Path
 from subprocess import CalledProcessError
 
-import requests
-from ghapi.core import GhApi
-from pkg_resources import parse_version
+import mdformat
+from packaging.version import parse as parse_version
+from pkginfo import SDist, Wheel
 
-from jupyter_releaser import changelog
-from jupyter_releaser import npm
-from jupyter_releaser import python
-from jupyter_releaser import util
+from jupyter_releaser import changelog, npm, python, util
 
 
-def bump_version(version_spec, version_cmd):
+def bump_version(version_spec, version_cmd, changelog_path):
     """Bump the version and verify new version"""
-    util.bump_version(version_spec, version_cmd=version_cmd)
+    util.bump_version(version_spec, version_cmd=version_cmd, changelog_path=changelog_path)
 
     version = util.get_version()
 
@@ -42,101 +41,91 @@ def bump_version(version_spec, version_cmd):
     return version
 
 
-def check_links(ignore_glob, ignore_links, cache_file, links_expire):
-    """Check URLs for HTML-containing files."""
-    cache_dir = osp.expanduser(cache_file).replace(os.sep, "/")
-    os.makedirs(cache_dir, exist_ok=True)
-    cmd = "pytest --noconftest --check-links --check-links-cache "
-    cmd += f"--check-links-cache-expire-after {links_expire} "
-    cmd += "--disable-warnings --quiet "
-    cmd += f"--check-links-cache-name {cache_dir}/check-release-links "
-
-    ignored = []
-    for spec in ignore_glob:
-        cmd += f' --ignore-glob "{spec}"'
-        ignored.extend(glob(spec, recursive=True))
-
-    ignore_links = list(ignore_links) + [
-        "https://github.com/.*/(pull|issues)/.*",
-        "https://github.com/search?",
-        "http://localhost.*",
-    ]
-
-    for spec in ignore_links:
-        cmd += f' --check-links-ignore "{spec}"'
-
-    cmd += " --ignore-glob node_modules"
-
-    # Gather all of the markdown, RST, and ipynb files
-    files = []
-    for ext in [".md", ".rst", ".ipynb"]:
-        matched = glob(f"**/*{ext}", recursive=True)
-        files.extend(m for m in matched if not m in ignored and "node_modules" not in m)
-
-    for f in files:
-        file_cmd = cmd + f' "{f}"'
-        try:
-            util.run(file_cmd, shell=False)
-        except Exception as e:
-            # Return code 5 means no tests were run (no links found)
-            if e.returncode != 5:
-                util.run(file_cmd + " --lf", shell=False)
-
-
 def draft_changelog(
-    version_spec, branch, repo, since, since_last_stable, auth, changelog_path, dry_run
+    version_spec,
+    ref,
+    branch,
+    repo,
+    since,
+    since_last_stable,
+    auth,
+    changelog_path,
+    dry_run,
+    post_version_spec,
+    post_version_message,
 ):
     """Create a changelog entry PR"""
     repo = repo or util.get_repo()
     branch = branch or util.get_branch()
     version = util.get_version()
+    prerelease = util.is_prerelease(version)
+
+    current_sha = util.run("git rev-parse HEAD")
 
     # Check for multiple versions
     npm_versions = None
     if util.PACKAGE_JSON.exists():
         npm_versions = npm.get_package_versions(version)
+        util.log(npm_versions)
 
     tags = util.run("git --no-pager tag", quiet=True)
     if f"v{version}" in tags.splitlines():
         raise ValueError(f"Tag v{version} already exists")
 
-    # Check out any unstaged files from version bump
-    # If this is an automated changelog PR, there may be no effective changes
+    current = changelog.extract_current(changelog_path)
+    util.log(f"\n\nCurrent Changelog Entry:\n{current}")
+
+    # Check out all changed files.
     try:
-        util.run("git checkout -- .")
+        util.run("git checkout .", echo=True)
     except CalledProcessError as e:
         util.log(str(e))
         return
 
-    current = changelog.extract_current(changelog_path)
-    util.log(f"\n\nCurrent Changelog Entry:\n{current}")
+    util.run("git status", echo=True)
+    util.log(f"\n\nCreating draft GitHub release for {version}")
+    owner, repo_name = repo.split("/")
+    gh = util.get_gh_object(dry_run=dry_run, owner=owner, repo=repo_name, token=auth)
 
-    title = f"{changelog.PR_PREFIX} for {version} on {branch}"
-    commit_message = f'git commit -a -m "{title}"'
-    body = title
+    data = dict(
+        version_spec=version_spec,
+        ref=ref,
+        branch=branch,
+        repo=repo,
+        since=since,
+        since_last_stable=since_last_stable,
+        version=version,
+        post_version_spec=post_version_spec,
+        post_version_message=post_version_message,
+        expected_sha=current_sha,
+    )
+    with tempfile.TemporaryDirectory() as d:
+        metadata_path = Path(d) / util.METADATA_JSON
+        with open(metadata_path, "w") as fid:
+            json.dump(data, fid)
 
-    if npm_versions:
-        body += npm_versions
+        release = gh.create_release(
+            f"v{version}", branch, f"v{version}", current, True, prerelease, files=[metadata_path]
+        )
 
-    body += '\n\nAfter merging this PR run the "Full Release" Workflow on your fork of `jupyter_releaser` with the following inputs'
-    body += f"""
-| Input  | Value |
-| ------------- | ------------- |
-| Target | {repo}  |
-| Branch  | {branch}  |
-| Version Spec | {version_spec} |
-"""
-    if since_last_stable:
-        body += "| Since Last Stable | true |"
-    elif since:
-        body += f"| Since | {since} |"
-    util.log(body)
+    # Remove draft releases over a day old
+    if bool(os.environ.get("GITHUB_ACTIONS")):
+        for rel in gh.repos.list_releases():
+            if str(rel.draft).lower() == "false":
+                continue
+            created = rel.created_at
+            d_created = datetime.strptime(created, r"%Y-%m-%dT%H:%M:%SZ")
+            delta = datetime.utcnow() - d_created
+            if delta.days > 0:
+                gh.repos.delete_release(rel.id)
 
-    make_changelog_pr(auth, branch, repo, title, commit_message, body, dry_run=dry_run)
+    # Set the GitHub action output for the release url.
+    util.actions_output("release_url", release.html_url)
 
 
 def make_changelog_pr(auth, branch, repo, title, commit_message, body, dry_run=False):
     repo = repo or util.get_repo()
+    branch = branch or util.get_branch()
 
     # Make a new branch with a uuid suffix
     pr_branch = f"changelog-{uuid.uuid1().hex}"
@@ -145,7 +134,7 @@ def make_changelog_pr(auth, branch, repo, title, commit_message, body, dry_run=F
         dirty = util.run("git --no-pager diff --stat") != ""
         if dirty:
             util.run("git stash")
-        util.run(f"git fetch origin {branch}")
+        util.run(f"{util.GIT_FETCH_CMD} {branch}")
         util.run(f"git checkout -b {pr_branch} origin/{branch}")
         if dirty:
             util.run("git stash apply")
@@ -159,27 +148,29 @@ def make_changelog_pr(auth, branch, repo, title, commit_message, body, dry_run=F
 
     # Create the pull
     owner, repo_name = repo.split("/")
-    gh = GhApi(owner=owner, repo=repo_name, token=auth)
+    gh = util.get_gh_object(dry_run=dry_run, owner=owner, repo=repo_name, token=auth)
 
     base = branch
     head = pr_branch
     maintainer_can_modify = True
 
-    if dry_run:
-        util.log("Skipping pull request due to dry run")
-        return
-
-    util.run(f"git push origin {pr_branch}")
+    if not dry_run:
+        util.run(f"git push origin {pr_branch}", echo=True)
 
     #  title, head, base, body, maintainer_can_modify, draft, issue
     pull = gh.pulls.create(title, head, base, body, maintainer_can_modify, False, None)
 
+    # Try to add the documentation label to the PR.
+    number = pull.number
+    try:
+        gh.issues.add_labels(number, ["documentation"])
+    except Exception as e:
+        print(e)
+
     util.actions_output("pr_url", pull.html_url)
 
 
-def tag_release(
-    dist_dir, release_message, tag_format, tag_message, no_git_tag_workspace
-):
+def tag_release(dist_dir, release_message, tag_format, tag_message, no_git_tag_workspace):
     """Create release commit and tag"""
     # Get the new version
     version = util.get_version()
@@ -197,78 +188,72 @@ def tag_release(
         npm.tag_workspace_packages()
 
 
-def draft_release(
+def populate_release(
     ref,
     branch,
     repo,
+    version_cmd,
     auth,
     changelog_path,
-    version_cmd,
     dist_dir,
     dry_run,
+    release_url,
     post_version_spec,
     post_version_message,
     assets,
 ):
-    """Publish Draft GitHub release and handle post version bump"""
+    """Populate release assets and push tags and commits"""
     branch = branch or util.get_branch()
-    repo = repo or util.get_repo()
     assets = assets or glob(f"{dist_dir}/*")
-    version = util.get_version()
     body = changelog.extract_current(changelog_path)
-    prerelease = util.is_prerelease(version)
 
-    # Bump to post version if given
+    match = util.parse_release_url(release_url)
+    owner, repo_name = match["owner"], match["repo"]
+
+    # Bump to post version if given.
     if post_version_spec:
-        post_version = bump_version(post_version_spec, version_cmd)
+        post_version = bump_version(
+            post_version_spec, version_cmd=version_cmd, changelog_path=changelog_path
+        )
         util.log(post_version_message.format(post_version=post_version))
         util.run(f'git commit -a -m "Bump to {post_version}"')
 
-    if dry_run:
-        return
+    gh = util.get_gh_object(dry_run=dry_run, owner=owner, repo=repo_name, token=auth)
+    release = util.release_for_url(gh, release_url)
 
-    owner, repo_name = repo.split("/")
-    gh = GhApi(owner=owner, repo=repo_name, token=auth)
-
-    # Remove draft releases over a day old
-    if bool(os.environ.get("GITHUB_ACTIONS")):
-        for release in gh.repos.list_releases():
-            if str(release.draft).lower() == "false":
-                continue
-            created = release.created_at
-            d_created = datetime.strptime(created, r"%Y-%m-%dT%H:%M:%SZ")
-            delta = datetime.utcnow() - d_created
-            if delta.days > 0:
-                gh.repos.delete_release(release.id)
-
-    remote_url = util.run("git config --get remote.origin.url")
+    remote_name = util.get_remote_name(dry_run)
+    remote_url = util.run(f"git config --get remote.{remote_name}.url")
     if not os.path.exists(remote_url):
-        util.run(f"git push origin HEAD:{branch} --follow-tags --tags")
+        util.run(f"git push {remote_name} HEAD:{branch} --follow-tags --tags")
 
-    util.log(f"Creating release for {version}")
-    util.log(f"With assets: {assets}")
-    release = gh.create_release(
-        f"v{version}",
-        branch,
-        f"v{version}",
+    # Set the body of the release with the changelog contents.
+    # Get the new release since the draft release might change urls.
+    release = gh.repos.update_release(
+        release.id,
+        release.tag_name,
+        release.target_commitish,
+        release.name,
         body,
         True,
-        prerelease,
-        files=assets,
+        release.prerelease,
     )
 
+    # Upload the assets to the draft release.
+    release = util.upload_assets(gh, assets, release, auth)
+
     # Set the GitHub action output
     util.actions_output("release_url", release.html_url)
 
 
-def delete_release(auth, release_url):
+def delete_release(auth, release_url, dry_run=False):
     """Delete a draft GitHub release by url to the release page"""
-    match = re.match(util.RELEASE_HTML_PATTERN, release_url)
+    pattern = util.RELEASE_HTML_PATTERN % util.get_mock_github_url()
+    match = re.match(pattern, release_url)
     match = match or re.match(util.RELEASE_API_PATTERN, release_url)
     if not match:
         raise ValueError(f"Release url is not valid: {release_url}")
 
-    gh = GhApi(owner=match["owner"], repo=match["repo"], token=auth)
+    gh = util.get_gh_object(dry_run=dry_run, owner=match["owner"], repo=match["repo"], token=auth)
     release = util.release_for_url(gh, release_url)
     for asset in release.assets:
         gh.repos.delete_release_asset(asset.id)
@@ -276,11 +261,12 @@ def delete_release(auth, release_url):
     gh.repos.delete_release(release.id)
 
 
-def extract_release(auth, dist_dir, dry_run, release_url, npm_install_options):
+def extract_release(auth, dist_dir, dry_run, release_url):
     """Download and verify assets from a draft GitHub release"""
-    match = parse_release_url(release_url)
+    match = util.parse_release_url(release_url)
     owner, repo = match["owner"], match["repo"]
-    gh = GhApi(owner=owner, repo=repo, token=auth)
+
+    gh = util.get_gh_object(dry_run=dry_run, owner=owner, repo=repo, token=auth)
     release = util.release_for_url(gh, release_url)
     branch = release.target_commitish
     assets = release.assets
@@ -291,11 +277,6 @@ def extract_release(auth, dist_dir, dry_run, release_url, npm_install_options):
     orig_dir = os.getcwd()
     os.chdir(util.CHECKOUT_NAME)
 
-    # Read in the config
-    config = util.read_config()
-    options = config.get("options", {})
-    npm_install_options = npm_install_options or options.get("npm-install-options", "")
-
     # Clean the dist folder
     dist = Path(dist_dir)
     if dist.exists():
@@ -304,135 +285,110 @@ def extract_release(auth, dist_dir, dry_run, release_url, npm_install_options):
 
     # Fetch, validate, and publish assets
     for asset in assets:
-        util.log(f"Fetching {asset.name}...")
-        url = asset.url
-        headers = dict(Authorization=f"token {auth}", Accept="application/octet-stream")
-        path = dist / asset.name
-        with requests.get(url, headers=headers, stream=True) as r:
-            r.raise_for_status()
-            with open(path, "wb") as f:
-                for chunk in r.iter_content(chunk_size=8192):
-                    f.write(chunk)
+        util.fetch_release_asset(dist_dir, asset, auth)
 
-    # Check all npm packages
-    npm.check_dist(dist, npm_install_options)
-
-    # Check python packages individually
-    for asset in assets:
-        suffix = Path(asset.name).suffix
-        if suffix in [".gz", ".whl"]:
-            python.check_dist(dist / asset.name)
-        elif suffix == ".tgz":
-            pass  # already handled
-        else:
-            util.log(f"Nothing to check for {asset.name}")
-
-    # Skip sha validation for dry runs since the remote tag will not exist
-    if dry_run:
-        return
-
-    tag_name = release.tag_name
-
-    sha = None
-    for tag in gh.list_tags():
-        if tag.ref == f"refs/tags/{tag_name}":
-            sha = tag.object.sha
-    if sha is None:
-        raise ValueError("Could not find tag")
-
-    # Get the commmit message for the tag
-    commit_message = ""
-    commit_message = util.run(f"git log --format=%B -n 1 {sha}")
+    # Validate the shas of all the files
+    asset_shas_file = dist / "asset_shas.json"
+    with open(asset_shas_file) as fid:
+        asset_shas = json.load(fid)
+    asset_shas_file.unlink()
 
     for asset in assets:
-        # Check the sha against the published sha
-        valid = False
-        path = dist / asset.name
-        sha = util.compute_sha256(path)
-
-        for line in commit_message.splitlines():
-            if asset.name in line:
-                if sha in line:
-                    valid = True
-                else:
-                    util.log("Mismatched sha!")
-
-        if not valid:  # pragma: no cover
-            raise ValueError(f"Invalid file {asset.name}")
+        if asset.name.endswith(".json"):
+            continue
+        if asset.name not in asset_shas:
+            raise ValueError(f"{asset.name} was not found in asset_shas file")
+        if util.compute_sha256(dist / asset.name) != asset_shas[asset.name]:
+            raise ValueError(f"sha for {asset.name} does not match asset_shas file")
 
     os.chdir(orig_dir)
 
 
-def parse_release_url(release_url):
-    """Parse a release url into a regex match"""
-    match = re.match(util.RELEASE_HTML_PATTERN, release_url)
-    match = match or re.match(util.RELEASE_API_PATTERN, release_url)
-    if not match:
-        raise ValueError(f"Release url is not valid: {release_url}")
-    return match
-
-
 def publish_assets(
-    dist_dir, npm_token, npm_cmd, twine_cmd, npm_registry, twine_registry, dry_run
+    dist_dir,
+    npm_token,
+    npm_cmd,
+    twine_cmd,
+    npm_registry,
+    twine_repository_url,
+    dry_run,
+    release_url,
+    python_package,
 ):
     """Publish release asset(s)"""
     os.environ["NPM_REGISTRY"] = npm_registry
-    os.environ["TWINE_REGISTRY"] = twine_registry
-
-    if dry_run:
-        # Start local pypi server with no auth, allowing overwrites,
-        # in a temporary directory
-        if len(glob(f"{dist_dir}/*.whl")):
-            python.start_local_pypi()
-            twine_cmd = "twine upload --repository-url=http://0.0.0.0:8081"
-            os.environ["TWINE_USERNAME"] = "foo"
-            os.environ["TWINE_PASSWORD"] = "bar"
-        npm_cmd = "npm publish --dry-run"
-    else:
-        os.environ.setdefault("TWINE_USERNAME", "__token__")
+    os.environ["TWINE_REPOSITORY_URL"] = twine_repository_url
+    twine_token = ""
 
     if len(glob(f"{dist_dir}/*.tgz")):
         npm.handle_npm_config(npm_token)
         if npm_token:
             util.run("npm whoami")
 
+    res = python_package.split(":")
+    python_package_path = res[0]
+    if len(res) == 2:
+        python_package_name = res[1].replace("-", "_")
+    else:
+        python_package_name = ""
+
+    if release_url and len(glob(f"{dist_dir}/*.whl")):
+        twine_token = python.get_pypi_token(release_url, python_package_path)
+
+    if dry_run:
+        # Start local pypi server with no auth, allowing overwrites,
+        # in a temporary directory
+        if len(glob(f"{dist_dir}/*.whl")):
+            python.start_local_pypi()
+            twine_cmd = "pipx run twine upload --repository-url=http://0.0.0.0:8081"
+            os.environ["TWINE_USERNAME"] = "foo"
+            twine_token = twine_token or "bar"
+        npm_cmd = "npm publish --dry-run"
+    else:
+        os.environ.setdefault("TWINE_USERNAME", "__token__")
+
     found = False
     for path in sorted(glob(f"{dist_dir}/*.*")):
         name = Path(path).name
+        util.log(f"Handling dist file {path}")
         suffix = Path(path).suffix
         if suffix in [".gz", ".whl"]:
-            util.retry(f"{twine_cmd} {name}", cwd=dist_dir)
-            found = True
+            if suffix == ".gz":
+                dist = SDist
+            else:
+                dist = Wheel
+            pkg = dist(path)
+            if not python_package_name or python_package_name == pkg.name:
+                env = os.environ.copy()
+                env["TWINE_PASSWORD"] = twine_token
+                # NOTE: Do not print the env since a twine token extracted from
+                # a PYPI_TOKEN_MAP will not be sanitized in output
+                util.retry(f"{twine_cmd} {name}", cwd=dist_dir, env=env, echo=True)
+                found = True
         elif suffix == ".tgz":
             # Ignore already published versions
             try:
-                util.run(
-                    f"{npm_cmd} {name}", cwd=dist_dir, quiet=True, quiet_error=True
-                )
+                util.run(f"{npm_cmd} {name}", cwd=dist_dir, quiet=True, quiet_error=True, echo=True)
             except CalledProcessError as e:
                 stderr = e.stderr
-                if (
-                    "EPUBLISHCONFLICT" in stderr
-                    or "previously published versions" in stderr
-                ):
+                if "EPUBLISHCONFLICT" in stderr or "previously published versions" in stderr:
                     continue
                 raise e
             found = True
         else:
             util.log(f"Nothing to upload for {name}")
-
-    if not found:  # pragma: no cover
-        raise ValueError("No assets published, refusing to finalize release")
+    if not found:
+        util.log("No files to upload")
 
 
-def publish_release(auth, release_url):
+def publish_release(auth, dry_run, release_url):
     """Publish GitHub release"""
     util.log(f"Publishing {release_url}")
 
-    match = parse_release_url(release_url)
+    match = util.parse_release_url(release_url)
 
     # Take the release out of draft
-    gh = GhApi(owner=match["owner"], repo=match["repo"], token=auth)
+    gh = util.get_gh_object(dry_run=dry_run, owner=match["owner"], repo=match["repo"], token=auth)
     release = util.release_for_url(gh, release_url)
 
     release = gh.repos.update_release(
@@ -453,20 +409,6 @@ def prep_git(ref, branch, repo, auth, username, url):
     """Set up git"""
     repo = repo or util.get_repo()
 
-    user_name = ""
-    try:
-        user_name = util.run("git config --global user.email")
-    except Exception:
-        pass
-
-    if not user_name:
-        # Use email address for the GitHub Actions bot
-        # https://github.community/t/github-actions-bot-email-address/17204/6
-        util.run(
-            'git config --global user.email "41898282+github-actions[bot]@users.noreply.github.com"'
-        )
-        util.run('git config --global user.name "GitHub Action"')
-
     # Set up the repository
     checkout_dir = os.environ.get("RH_CHECKOUT_DIR", util.CHECKOUT_NAME)
     checkout_exists = False
@@ -496,7 +438,7 @@ def prep_git(ref, branch, repo, auth, username, url):
     ref = ref or ""
 
     # Make sure we have *all* tags
-    util.run("git fetch origin --tags --force --quiet")
+    util.run(f"{util.GIT_FETCH_CMD} --tags --force")
 
     # Handle the ref
     if ref.startswith("refs/pull/"):
@@ -507,11 +449,11 @@ def prep_git(ref, branch, repo, auth, username, url):
 
     # Reuse existing branch if possible
     if ref:
-        util.run(f"git fetch origin +{ref}:{ref_alias}")
-        util.run(f"git fetch origin {ref}")
+        util.run(f"{util.GIT_FETCH_CMD} +{ref}:{ref_alias}")
+        util.run(f"{util.GIT_FETCH_CMD} {ref}")
         checkout_cmd = f"git checkout -B {branch} {ref_alias}"
     else:
-        util.run(f"git fetch origin {branch}")
+        util.run(f"{util.GIT_FETCH_CMD} {branch}")
         checkout_cmd = f"git checkout {branch}"
 
     if checkout_exists:
@@ -528,33 +470,38 @@ def prep_git(ref, branch, repo, auth, username, url):
     except Exception:
         util.run(f"git switch -c {branch}")
 
-    # Install the package
-    # install python package in editable mode with test deps
-    if util.SETUP_PY.exists():
-        util.run('pip install -q -e ".[test]"')
+    try:
+        has_git_config = util.run("git config user.email").strip()
+    except Exception:
+        has_git_config = False
 
-    # prefer yarn if yarn lock exists
-    elif util.YARN_LOCK.exists():
-        if not shutil.which("yarn"):
-            util.run("npm install -g yarn")
-        util.run("yarn")
-
-    # npm install otherwise
-    elif util.PACKAGE_JSON.exists():
-        util.run("npm install")
+    if not has_git_config:
+        # Default to the GitHub Actions bot
+        # https://github.community/t/github-actions-bot-email-address/17204/6
+        git_user_name = username or "41898282+github-actions[bot]"
+        util.run(f'git config user.email "{git_user_name}@users.noreply.github.com"', echo=True)
+        util.run(f'git config user.name "{git_user_name}"', echo=True)
 
     os.chdir(orig_dir)
 
     return branch
 
 
-def forwardport_changelog(
-    auth, ref, branch, repo, username, changelog_path, dry_run, release_url
-):
+def extract_changelog(dry_run, auth, changelog_path, release_url):
+    """Extract the changelog from the draft GH release body and update it."""
+    match = util.parse_release_url(release_url)
+    gh = util.get_gh_object(dry_run=dry_run, owner=match["owner"], repo=match["repo"], token=auth)
+    release = util.release_for_url(gh, release_url)
+    changelog_text = mdformat.text(release.body)
+    changelog.update_changelog(changelog_path, changelog_text)
+
+
+def forwardport_changelog(auth, ref, branch, repo, username, changelog_path, dry_run, release_url):
     """Forwardport Changelog Entries to the Default Branch"""
     # Set up the git repo with the branch
-    match = parse_release_url(release_url)
-    gh = GhApi(owner=match["owner"], repo=match["repo"], token=auth)
+    match = util.parse_release_url(release_url)
+
+    gh = util.get_gh_object(dry_run=dry_run, owner=match["owner"], repo=match["repo"], token=auth)
     release = util.release_for_url(gh, release_url)
     tag = release.tag_name
     source_branch = release.target_commitish
@@ -563,7 +510,7 @@ def forwardport_changelog(
 
     # switch to main branch here
     branch = branch or util.get_default_branch()
-    util.run(f"git fetch origin {branch}")
+    util.run(f"{util.GIT_FETCH_CMD} {branch}")
     util.run(f"git checkout {branch}")
 
     # Bail if the tag has been merged to the branch
@@ -596,7 +543,7 @@ def forwardport_changelog(
 
     # Look for the previous header
     default_log = Path(changelog_path).read_text(encoding="utf-8")
-    if not prev_header in default_log:
+    if prev_header not in default_log:
         util.log(
             f'Could not find previous header "{prev_header}" in {changelog_path} on branch {branch}'
         )
@@ -620,9 +567,7 @@ def forwardport_changelog(
     commit_message = f'git commit -a -m "{title}"'
     body = title
 
-    pr = make_changelog_pr(
-        auth, branch, repo, title, commit_message, body, dry_run=dry_run
-    )
+    make_changelog_pr(auth, branch, repo, title, commit_message, body, dry_run=dry_run)
 
     # Clean up after ourselves
     util.run(f"git checkout {source_branch}")
