@@ -89,26 +89,26 @@ def extract_dist(dist_dir, target):
 
 def check_dist(dist_dir, install_options):
     """Check npm dist file(s) in a dist dir"""
-    tmp_dir = Path(TemporaryDirectory().name)
-    os.makedirs(tmp_dir)
+    with TemporaryDirectory() as td:
 
-    util.run("npm init -y", cwd=tmp_dir, quiet=True)
-    names = []
-    staging = tmp_dir / "staging"
+        util.run("npm init -y", cwd=td, quiet=True)
+        names = []
+        staging = Path(td) / "staging"
 
-    names = extract_dist(dist_dir, staging)
+        names = extract_dist(dist_dir, staging)
 
-    install_str = " ".join(f"./staging/{name}" for name in names)
+        install_str = " ".join(f"./staging/{name}" for name in names)
 
-    util.run(f"npm install {install_options} {install_str}", cwd=tmp_dir, quiet=True)
-
-    shutil.rmtree(str(tmp_dir), ignore_errors=True)
+        util.run(f"npm install {install_options} {install_str}", cwd=td, quiet=True)
 
 
 def extract_package(path):
     """Get the package json info from the tarball"""
     fid = tarfile.open(path)
-    data = fid.extractfile("package/package.json").read()
+    fidfile = fid.extractfile("package/package.json")
+    assert fidfile is not None
+    data = fidfile.read()
+    fidfile.close()
     data = json.loads(data.decode("utf-8"))
     fid.close()
     return data
@@ -135,7 +135,7 @@ def handle_npm_config(npm_token):
 
     text += f"\n{reg_entry}\n{auth_entry}"
     text = text.strip() + "\n"
-    util.log(f"writing npm config to {npmrc}:\n{text}")
+    util.log(f"writing npm config to {npmrc}")
     npmrc.write_text(text, encoding="utf-8")
 
 
@@ -199,4 +199,4 @@ def _get_workspace_packages(data):
                 continue
             paths.append(sub_package)
 
-    return paths
+    return sorted(paths)
