@@ -3,16 +3,27 @@
 import json
 import os
 import os.path as osp
+import tempfile
+import time
+import uuid
 from pathlib import Path
-from urllib.request import OpenerDirector
 
 from click.testing import CliRunner
+from ghapi.core import GhApi
 from pytest import fixture
 
-from jupyter_releaser import cli
-from jupyter_releaser import util
+from jupyter_releaser import cli, util
 from jupyter_releaser.tests import util as testutil
-from jupyter_releaser.util import run
+from jupyter_releaser.util import ensure_mock_github, run
+
+
+@fixture(autouse=True)
+def github_port(worker_id):
+    # The worker id will be of the form "gw123" unless xdist is disabled,
+    # in which case it will be "master".
+    if worker_id == "master":
+        return
+    os.environ["MOCK_GITHUB_PORT"] = str(8000 + int(worker_id[2:]))
 
 
 @fixture(autouse=True)
@@ -27,13 +38,6 @@ def mock_env(mocker):
                 del env[key]
 
     mocker.patch.dict(os.environ, env, clear=True)
-
-    try:
-        run("git config --global user.name")
-    except Exception:
-        run("git config --global user.name snuffy")
-        run("git config --global user.email snuffy@sesame.com")
-
     yield
 
 
@@ -53,9 +57,6 @@ def git_repo(tmp_path):
     changelog = tmp_path / "CHANGELOG.md"
     changelog.write_text(testutil.CHANGELOG_TEMPLATE, encoding="utf-8")
 
-    readme = tmp_path / "README.md"
-    readme.write_text("Hello from foo project\n", encoding="utf-8")
-
     config = Path(util.JUPYTER_RELEASER_CONFIG)
     config.write_text(testutil.TOML_CONFIG, encoding="utf-8")
 
@@ -75,6 +76,16 @@ def py_package(git_repo):
     return testutil.create_python_package(git_repo)
 
 
+@fixture
+def py_multipackage(git_repo):
+    return testutil.create_python_package(git_repo, multi=True)
+
+
+@fixture
+def py_package_different_names(git_repo):
+    return testutil.create_python_package(git_repo, not_matching_name=True)
+
+
 @fixture
 def npm_package(git_repo):
     return testutil.create_npm_package(git_repo)
@@ -119,7 +130,7 @@ def py_dist(py_package, runner, mocker, build_mock, git_prep):
     changelog_entry = testutil.mock_changelog_entry(py_package, runner, mocker)
 
     # Create the dist files
-    util.run("python -m build .", cwd=util.CHECKOUT_NAME, quiet=True)
+    util.run("pipx run build .", cwd=util.CHECKOUT_NAME, quiet=True)
 
     # Finalize the release
     runner(["tag-release"])
@@ -150,6 +161,7 @@ def runner():
             if result.stderr_bytes:
                 print("Captured stderr\n", result.stderr, "\n\n")
             print("Catpured stdout\n", result.stdout, "\n\n")
+            assert result.exception is not None
             raise result.exception
 
         return result
@@ -162,29 +174,60 @@ def git_prep(runner, git_repo):
     runner(["prep-git", "--git-url", git_repo])
 
 
-@fixture
-def open_mock(mocker):
-    open_mock = mocker.patch.object(OpenerDirector, "open", autospec=True)
-    open_mock.return_value = testutil.MockHTTPResponse()
-    yield open_mock
-
-
 @fixture
 def build_mock(mocker):
     orig_run = util.run
 
     def wrapped(cmd, **kwargs):
-        if cmd == "python -m build .":
+        if cmd == "pipx run build .":
             if osp.exists(util.CHECKOUT_NAME):
                 dist_dir = Path(f"{util.CHECKOUT_NAME}/dist")
             else:
                 dist_dir = Path("dist")
             os.makedirs(dist_dir, exist_ok=True)
             Path(f"{dist_dir}/foo-0.0.2a0.tar.gz").write_text("hello", encoding="utf-8")
-            Path(f"{dist_dir}/foo-0.0.2a0-py3-none-any.whl").write_text(
-                "hello", encoding="utf-8"
-            )
+            Path(f"{dist_dir}/foo-0.0.2a0-py3-none-any.whl").write_text("hello", encoding="utf-8")
             return ""
         return orig_run(cmd, **kwargs)
 
     mock_run = mocker.patch("jupyter_releaser.util.run", wraps=wrapped)
+
+
+@fixture
+def mock_github():
+    proc = ensure_mock_github()
+    yield proc
+
+    if proc:
+        proc.kill()
+        proc.wait()
+
+
+@fixture
+def draft_release(mock_github):
+    gh = GhApi(owner="foo", repo="bar")
+    tag = uuid.uuid4().hex
+    data = dict(
+        version_spec="foo",
+        branch="bar",
+        repo="fizz",
+        since="buzz",
+        since_last_stable=False,
+        version=tag,
+        post_version_spec="dev",
+        post_version_message="hi",
+    )
+
+    with tempfile.TemporaryDirectory() as d:
+        metadata_path = Path(d) / "metadata.json"
+        with open(metadata_path, "w") as fid:
+            json.dump(data, fid)
+
+        # Ensure this is the latest release.
+        time.sleep(1)
+        release = gh.create_release(tag, "bar", tag, "hi", True, True, files=[metadata_path])
+    yield release.html_url
+    try:
+        gh.repos.delete_release(release.id)
+    except Exception as e:
+        print(e)
