@@ -12,20 +12,16 @@ from unittest.mock import call
 
 import pytest
 
-from jupyter_releaser import changelog
-from jupyter_releaser import util
-from jupyter_releaser.tests.util import CHANGELOG_ENTRY
-from jupyter_releaser.tests.util import get_log
-from jupyter_releaser.tests.util import HTML_URL
-from jupyter_releaser.tests.util import mock_changelog_entry
-from jupyter_releaser.tests.util import MockHTTPResponse
-from jupyter_releaser.tests.util import MockRequestResponse
-from jupyter_releaser.tests.util import PR_ENTRY
-from jupyter_releaser.tests.util import REPO_DATA
-from jupyter_releaser.tests.util import VERSION_SPEC
-from jupyter_releaser.util import bump_version
-from jupyter_releaser.util import normalize_path
-from jupyter_releaser.util import run
+from jupyter_releaser import changelog, util
+from jupyter_releaser.tests.util import (
+    CHANGELOG_ENTRY,
+    PR_ENTRY,
+    VERSION_SPEC,
+    create_draft_release,
+    get_log,
+    mock_changelog_entry,
+)
+from jupyter_releaser.util import GIT_FETCH_CMD, normalize_path, run
 
 
 def test_prep_git_simple(py_package, runner):
@@ -100,17 +96,17 @@ def test_prep_git_full(py_package, tmp_path, mocker, runner):
     runner(["prep-git"], env=env)
     mock_run.assert_has_calls(
         [
-            call("git config --global user.email"),
-            call(
-                'git config --global user.email "41898282+github-actions[bot]@users.noreply.github.com"'
-            ),
-            call('git config --global user.name "GitHub Action"'),
+            call("echo before-prep-git >> 'log.txt'"),
             call("git init .jupyter_releaser_checkout"),
             call("git remote add origin https://snuffy:abc123@github.com/baz/bar.git"),
-            call("git fetch origin --tags --force --quiet"),
-            call("git fetch origin +refs/pull/42:refs/pull/42"),
-            call("git fetch origin refs/pull/42"),
+            call(f"{GIT_FETCH_CMD} --tags --force"),
+            call(f"{GIT_FETCH_CMD} +refs/pull/42:refs/pull/42"),
+            call(f"{GIT_FETCH_CMD} refs/pull/42"),
             call("git checkout -B foo refs/pull/42"),
+            call("git symbolic-ref -q HEAD"),
+            call("git config user.email"),
+            call('git config user.email "snuffy@users.noreply.github.com"', echo=True),
+            call('git config user.name "snuffy"', echo=True),
         ]
     )
 
@@ -131,9 +127,7 @@ def test_bump_version(npm_package, runner):
 def test_bump_version_bad_version(py_package, runner):
     runner(["prep-git", "--git-url", py_package])
     with pytest.raises(CalledProcessError):
-        runner(
-            ["bump-version", "--version-spec", "a1.0.1"], env=dict(GITHUB_ACTIONS="")
-        )
+        runner(["bump-version", "--version-spec", "a1.0.1"], env=dict(GITHUB_ACTIONS=""))
 
 
 def test_bump_version_tag_exists(py_package, runner):
@@ -150,20 +144,23 @@ def test_list_envvars(runner):
         == """
 auth: GITHUB_ACCESS_TOKEN
 branch: RH_BRANCH
-cache-file: RH_CACHE_FILE
 changelog-path: RH_CHANGELOG
+check-imports: RH_CHECK_IMPORTS
 dist-dir: RH_DIST_DIR
 dry-run: RH_DRY_RUN
-links-expire: RH_LINKS_EXPIRE
+expected-sha: RH_EXPECTED_SHA
 npm-cmd: RH_NPM_COMMAND
 npm-install-options: RH_NPM_INSTALL_OPTIONS
 npm-registry: NPM_REGISTRY
 npm-token: NPM_TOKEN
-output: RH_CHANGELOG_OUTPUT
 post-version-message: RH_POST_VERSION_MESSAGE
 post-version-spec: RH_POST_VERSION_SPEC
+pydist-check-cmd: RH_PYDIST_CHECK_CMD
+pydist-resource-paths: RH_PYDIST_RESOURCE_PATHS
+python-packages: RH_PYTHON_PACKAGES
 ref: RH_REF
 release-message: RH_RELEASE_MESSAGE
+release-url: RH_RELEASE_URL
 repo: RH_REPOSITORY
 resolve-backports: RH_RESOLVE_BACKPORTS
 since: RH_SINCE
@@ -171,7 +168,7 @@ since-last-stable: RH_SINCE_LAST_STABLE
 tag-format: RH_TAG_FORMAT
 tag-message: RH_TAG_MESSAGE
 twine-cmd: TWINE_COMMAND
-twine-registry: TWINE_REGISTRY
+twine-repository-url: TWINE_REPOSITORY_URL
 username: GITHUB_ACTOR
 version-cmd: RH_VERSION_COMMAND
 version-spec: RH_VERSION_SPEC
@@ -182,20 +179,20 @@ version-spec: RH_VERSION_SPEC
 def test_build_changelog(py_package, mocker, runner):
     run("pre-commit run -a")
 
-    changelog_path = "CHANGELOG.md"
+    changelog_path_str = "CHANGELOG.md"
 
     runner(["prep-git", "--git-url", py_package])
     runner(["bump-version", "--version-spec", VERSION_SPEC])
 
     mocked_gen = mocker.patch("jupyter_releaser.changelog.generate_activity_md")
     mocked_gen.return_value = CHANGELOG_ENTRY
-    runner(["build-changelog", "--changelog-path", changelog_path])
+    runner(["build-changelog", "--changelog-path", changelog_path_str])
 
     log = get_log()
     assert "before-build-changelog" in log
     assert "after-build-changelog" in log
 
-    changelog_path = Path(util.CHECKOUT_NAME) / "CHANGELOG.md"
+    changelog_path = Path(util.CHECKOUT_NAME) / changelog_path_str
     text = changelog_path.read_text(encoding="utf-8")
     assert changelog.START_MARKER in text
     assert changelog.END_MARKER in text
@@ -230,7 +227,7 @@ def test_build_changelog_existing(py_package, mocker, runner):
 
     text = changelog_path.read_text(encoding="utf-8")
     assert "Definining contributions" in text, text
-    assert not "defining contributions" in text, text
+    assert "defining contributions" not in text, text
 
     assert len(re.findall(changelog.START_MARKER, text)) == 1
     assert len(re.findall(changelog.END_MARKER, text)) == 1
@@ -238,19 +235,15 @@ def test_build_changelog_existing(py_package, mocker, runner):
     run("pre-commit run -a", cwd=util.CHECKOUT_NAME)
 
 
-def test_build_changelog_backport(py_package, mocker, runner, open_mock):
+def test_build_changelog_backport(py_package, mocker, runner, mock_github):
     changelog_file = "CHANGELOG.md"
     changelog_path = Path(util.CHECKOUT_NAME) / changelog_file
 
-    data = dict(title="foo", html_url="bar", user=dict(login="snuffy", html_url="baz"))
-    open_mock.return_value = MockHTTPResponse(data)
-
     runner(["prep-git", "--git-url", py_package])
     runner(["bump-version", "--version-spec", VERSION_SPEC])
 
-    entry = CHANGELOG_ENTRY.replace("consideRatio", "meeseeksmachine")
-    entry = entry.replace(
-        "Support git references etc.", "Backport PR #50 (original title"
+    entry = CHANGELOG_ENTRY.replace(
+        "Support git references etc.", "Backport PR #50 on branch (original title"
     )
 
     mocked_gen = mocker.patch("jupyter_releaser.changelog.generate_activity_md")
@@ -260,7 +253,7 @@ def test_build_changelog_backport(py_package, mocker, runner, open_mock):
     assert changelog.START_MARKER in text
     assert changelog.END_MARKER in text
 
-    assert "- foo [#50](bar) ([@snuffy](baz))" in text, text
+    assert "- foo [#50](http://foo.com) ([@bar](http://bar.com))" in text, text
 
     assert len(re.findall(changelog.START_MARKER, text)) == 1
     assert len(re.findall(changelog.END_MARKER, text)) == 1
@@ -268,26 +261,26 @@ def test_build_changelog_backport(py_package, mocker, runner, open_mock):
     run("pre-commit run -a")
 
 
-def test_build_changelog_slashes(py_package, mocker, runner, open_mock):
+def test_build_changelog_slashes(py_package, mocker, runner):
     branch = "a/b/c"
     util.run(f"git checkout -b {branch} foo")
     env = dict(RH_REF=f"refs/heads/{branch}", RH_BRANCH=branch)
     run("pre-commit run -a")
 
-    changelog_path = "CHANGELOG.md"
+    changelog_path_str = "CHANGELOG.md"
 
     runner(["prep-git", "--git-url", py_package], env=env)
     runner(["bump-version", "--version-spec", VERSION_SPEC], env=env)
 
     mocked_gen = mocker.patch("jupyter_releaser.changelog.generate_activity_md")
     mocked_gen.return_value = CHANGELOG_ENTRY
-    runner(["build-changelog", "--changelog-path", changelog_path], env=env)
+    runner(["build-changelog", "--changelog-path", changelog_path_str], env=env)
 
     log = get_log()
     assert "before-build-changelog" in log
     assert "after-build-changelog" in log
 
-    changelog_path = Path(util.CHECKOUT_NAME) / "CHANGELOG.md"
+    changelog_path = Path(util.CHECKOUT_NAME) / changelog_path_str
     text = changelog_path.read_text(encoding="utf-8")
     assert changelog.START_MARKER in text
     assert changelog.END_MARKER in text
@@ -299,7 +292,7 @@ def test_build_changelog_slashes(py_package, mocker, runner, open_mock):
     run("pre-commit run -a")
 
 
-def test_draft_changelog_full(py_package, mocker, runner, open_mock, git_prep):
+def test_draft_changelog_full(py_package, mocker, runner, git_prep, mock_github):
     mock_changelog_entry(py_package, runner, mocker)
     runner(["draft-changelog", "--version-spec", VERSION_SPEC])
 
@@ -307,10 +300,8 @@ def test_draft_changelog_full(py_package, mocker, runner, open_mock, git_prep):
     assert "before-draft-changelog" in log
     assert "after-draft-changelog" in log
 
-    open_mock.assert_called_once()
 
-
-def test_draft_changelog_skip(py_package, mocker, runner, open_mock, git_prep):
+def test_draft_changelog_skip_config(py_package, mocker, runner, git_prep):
     mock_changelog_entry(py_package, runner, mocker)
 
     config_path = Path(util.CHECKOUT_NAME) / util.JUPYTER_RELEASER_CONFIG
@@ -319,7 +310,18 @@ def test_draft_changelog_skip(py_package, mocker, runner, open_mock, git_prep):
     config_path.write_text(util.toml.dumps(config), encoding="utf-8")
 
     runner(["draft-changelog", "--version-spec", VERSION_SPEC, "--since", "foo"])
-    open_mock.assert_not_called()
+
+
+def test_draft_changelog_skip_environ(py_package, mocker, runner, git_prep):
+    mock_changelog_entry(py_package, runner, mocker)
+
+    config_path = Path(util.CHECKOUT_NAME) / util.JUPYTER_RELEASER_CONFIG
+    config = util.toml.loads(config_path.read_text(encoding="utf-8"))
+    os.environ["RH_STEPS_TO_SKIP"] = "draft-changelog,other-fake-step"
+    config_path.write_text(util.toml.dumps(config), encoding="utf-8")
+
+    runner(["draft-changelog", "--version-spec", VERSION_SPEC, "--since", "foo"])
+    del os.environ["RH_STEPS_TO_SKIP"]
 
 
 def test_draft_changelog_dry_run(npm_package, mocker, runner, git_prep):
@@ -337,62 +339,9 @@ def test_draft_changelog_dry_run(npm_package, mocker, runner, git_prep):
     del os.environ["RH_SINCE_LAST_STABLE"]
 
 
-def test_draft_changelog_lerna(workspace_package, mocker, runner, open_mock, git_prep):
+def test_draft_changelog_lerna(workspace_package, mocker, runner, mock_github, git_prep):
     mock_changelog_entry(workspace_package, runner, mocker)
     runner(["draft-changelog", "--version-spec", VERSION_SPEC])
-    open_mock.assert_called_once()
-
-
-def test_check_links(py_package, runner):
-    readme = Path("README.md")
-    text = readme.read_text(encoding="utf-8")
-    text += "\nhttps://apod.nasa.gov/apod/astropix.html"
-    readme.write_text(text, encoding="utf-8")
-
-    config = Path(util.JUPYTER_RELEASER_CONFIG)
-    config_data = util.toml.loads(config.read_text(encoding="utf-8"))
-    config_data["options"] = {"ignore-glob": ["FOO.md"]}
-    config.write_text(util.toml.dumps(config_data), encoding="utf-8")
-
-    util.run("git commit -a -m 'update files'")
-
-    runner(["prep-git", "--git-url", py_package])
-    runner(["check-links"])
-
-    log = get_log()
-    assert "before-check-links" in log
-    assert "after-check-links" in log
-
-    foo = Path(util.CHECKOUT_NAME) / "FOO.md"
-    foo.write_text("http://127.0.0.1:5555")
-
-    bar = Path(util.CHECKOUT_NAME) / "BAR BAZ.md"
-    bar.write_text("")
-
-    runner(["check-links"])
-
-
-def test_check_changelog(py_package, tmp_path, mocker, runner, git_prep):
-    changelog_entry = mock_changelog_entry(py_package, runner, mocker)
-    output = "output.md"
-
-    # prep the release
-    bump_version(VERSION_SPEC)
-
-    runner(
-        ["check-changelog", "--changelog-path", changelog_entry, "--output", output],
-    )
-
-    log = get_log()
-    assert "before-check-changelog" in log
-    assert "after-check-changelog" in log
-
-    output = Path(util.CHECKOUT_NAME) / output
-    assert PR_ENTRY in output.read_text(encoding="utf-8")
-    changelog_entry = Path(util.CHECKOUT_NAME) / changelog_entry
-    text = changelog_entry.read_text(encoding="utf-8")
-    assert f"{changelog.START_MARKER}\n\n## {VERSION_SPEC}" in text
-    assert changelog.END_MARKER in text
 
 
 def test_build_python(py_package, runner, build_mock, git_prep):
@@ -421,6 +370,33 @@ def test_check_python(py_package, runner, build_mock, git_prep):
     assert "after-check-python" in log
 
 
+def test_check_python_different_names(
+    monkeypatch, py_package_different_names, runner, build_mock, git_prep
+):
+    monkeypatch.setenv("RH_CHECK_IMPORTS", "foobar")
+
+
+def test_check_python_resource_path(monkeypatch, py_package, runner, build_mock, git_prep):
+    monkeypatch.setenv("RH_PYDIST_RESOURCE_PATHS", "foo/bar/baz.txt")
+
+    # Convert the package to use a package dir.
+    foo_dir = Path(util.CHECKOUT_NAME) / "foo"
+    foo_dir.mkdir()
+    shutil.move(Path(util.CHECKOUT_NAME) / "foo.py", foo_dir / "__init__.py")
+
+    bar_dir = foo_dir / "bar"
+    bar_dir.mkdir()
+    path = bar_dir / "baz.txt"
+    path.write_text("hello", encoding="utf-8")
+
+    runner(["build-python"])
+    runner(["check-python"])
+
+    log = get_log()
+    assert "before-check-python" in log
+    assert "after-check-python" in log
+
+
 def test_handle_npm(npm_package, runner, git_prep):
     runner(["build-npm"])
 
@@ -440,23 +416,11 @@ def test_handle_npm_lerna(workspace_package, runner, git_prep):
     runner(["check-npm"])
 
 
-def test_check_manifest(py_package, runner, git_prep):
-    runner(["check-manifest"])
-
-    log = get_log()
-    assert "before-check-manifest" in log
-    assert "after-check-manifest" in log
-
-
-def test_check_manifest_npm(npm_package, runner, git_prep):
-    runner(["check-manifest"])
-
-
 def test_tag_release(py_package, runner, build_mock, git_prep):
     # Bump the version
     runner(["bump-version", "--version-spec", VERSION_SPEC])
     # Create the dist files
-    util.run("python -m build .", cwd=util.CHECKOUT_NAME)
+    util.run("pipx run build .", cwd=util.CHECKOUT_NAME)
     # Tag the release
     runner(
         [
@@ -473,73 +437,42 @@ def test_tag_release(py_package, runner, build_mock, git_prep):
     assert "after-tag-release" in log
 
 
-def test_draft_release_dry_run(py_dist, mocker, runner, open_mock, git_prep):
+def test_populate_release_dry_run(py_dist, mocker, runner, git_prep, draft_release):
     # Publish the release - dry run
     runner(
         [
-            "draft-release",
+            "populate-release",
             "--dry-run",
             "--post-version-spec",
             "1.1.0.dev0",
             "--post-version-message",
             "haha",
+            "--release-url",
+            draft_release,
         ]
     )
-    open_mock.assert_not_called()
 
     log = get_log()
-    assert "before-draft-release" in log
-    assert "after-draft-release" in log
+    assert "before-populate-release" in log
+    assert "after-populate-release" in log
 
 
-def test_draft_release_final(npm_dist, runner, mocker, open_mock, git_prep):
-    open_mock.side_effect = [
-        MockHTTPResponse([REPO_DATA]),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-    ]
-
+def test_populate_release_final(npm_dist, runner, mock_github, git_prep, draft_release):
     # Publish the release
     os.environ["GITHUB_ACTIONS"] = "true"
-    runner(["draft-release"])
-    assert len(open_mock.call_args) == 2
+    os.environ["RH_RELEASE_URL"] = draft_release
+    runner(["populate-release"])
 
 
-def test_delete_release(npm_dist, runner, mocker, open_mock, git_prep):
+def test_delete_release(npm_dist, runner, mock_github, git_prep, draft_release):
     # Publish the release
     # Mimic being on GitHub actions so we get the magic output
     os.environ["GITHUB_ACTIONS"] = "true"
-    open_mock.side_effect = [
-        MockHTTPResponse([REPO_DATA]),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-    ]
-    result = runner(["draft-release"])
-
-    assert len(open_mock.call_args) == 2
-
-    url = ""
-    for line in result.output.splitlines():
-        match = re.match(r"::set-output name=release_url::(.*)", line)
-        if match:
-            url = match.groups()[0]
+    os.environ["RH_RELEASE_URL"] = draft_release
+    result = runner(["populate-release"])
 
     # Delete the release
-    data = dict(assets=[dict(id="bar")])
-    open_mock.side_effect = [
-        MockHTTPResponse([data]),
-        MockHTTPResponse(),
-        MockHTTPResponse(),
-    ]
-    runner(["delete-release", url])
-    assert len(open_mock.call_args) == 2
+    runner(["delete-release"])
 
     log = get_log()
     assert "before-delete-release" in log
@@ -550,46 +483,22 @@ def test_delete_release(npm_dist, runner, mocker, open_mock, git_prep):
     os.name == "nt" and sys.version_info.major == 3 and sys.version_info.minor < 8,
     reason="See https://bugs.python.org/issue26660",
 )
-def test_extract_dist_py(py_package, runner, mocker, open_mock, tmp_path, git_prep):
+def test_extract_dist_py(py_package, runner, mocker, mock_github, tmp_path, git_prep):
     changelog_entry = mock_changelog_entry(py_package, runner, mocker)
 
     # Create the dist files
-    run("python -m build .", cwd=util.CHECKOUT_NAME)
+    run("pipx run build .", cwd=util.CHECKOUT_NAME)
 
     # Finalize the release
     runner(["tag-release"])
 
-    os.makedirs("staging")
-    shutil.move(f"{util.CHECKOUT_NAME}/dist", "staging")
+    # Create the release.
+    dist_dir = os.path.join(util.CHECKOUT_NAME, "dist")
+    release = create_draft_release("bar", glob(f"{dist_dir}/*.*"))
+    shutil.rmtree(dist_dir)
 
-    def helper(path, **kwargs):
-        return MockRequestResponse(f"{py_package}/staging/dist/{path}")
-
-    get_mock = mocker.patch("requests.get", side_effect=helper)
-
-    tag_name = f"v{VERSION_SPEC}"
-
-    dist_names = [osp.basename(f) for f in glob("staging/dist/*.*")]
-    releases = [
-        dict(
-            tag_name=tag_name,
-            target_commitish=util.get_branch(),
-            assets=[dict(name=dist_name, url=dist_name) for dist_name in dist_names],
-        )
-    ]
-    sha = run("git rev-parse HEAD", cwd=util.CHECKOUT_NAME)
-
-    tags = [dict(ref=f"refs/tags/{tag_name}", object=dict(sha=sha))]
-    url = normalize_path(osp.join(os.getcwd(), util.CHECKOUT_NAME))
-    open_mock.side_effect = [
-        MockHTTPResponse(releases),
-        MockHTTPResponse(tags),
-        MockHTTPResponse(dict(html_url=url)),
-    ]
-
-    runner(["extract-release", HTML_URL])
-    assert len(open_mock.mock_calls) == 2
-    assert len(get_mock.mock_calls) == len(dist_names) == 2
+    os.environ["RH_RELEASE_URL"] = release.html_url
+    runner(["extract-release"])
 
     log = get_log()
     assert "before-extract-release" not in log
@@ -600,37 +509,30 @@ def test_extract_dist_py(py_package, runner, mocker, open_mock, tmp_path, git_pr
     os.name == "nt" and sys.version_info.major == 3 and sys.version_info.minor < 8,
     reason="See https://bugs.python.org/issue26660",
 )
-def test_extract_dist_npm(npm_dist, runner, mocker, open_mock, tmp_path):
+def test_extract_dist_multipy(py_multipackage, runner, mocker, mock_github, tmp_path, git_prep):
+    git_repo = py_multipackage[0]["abs_path"]
+    changelog_entry = mock_changelog_entry(git_repo, runner, mocker)
 
-    os.makedirs("staging")
-    shutil.move(f"{util.CHECKOUT_NAME}/dist", "staging")
-
-    def helper(path, **kwargs):
-        return MockRequestResponse(f"{npm_dist}/staging/dist/{path}")
-
-    get_mock = mocker.patch("requests.get", side_effect=helper)
-
-    dist_names = [osp.basename(f) for f in glob("staging/dist/*.tgz")]
-    url = normalize_path(osp.join(os.getcwd(), util.CHECKOUT_NAME))
-    tag_name = f"v{VERSION_SPEC}"
-    releases = [
-        dict(
-            tag_name=tag_name,
-            target_commitish="foo",
-            assets=[dict(name=dist_name, url=dist_name) for dist_name in dist_names],
+    # Create the dist files
+    files = []
+    dist_dir = normalize_path(Path(util.CHECKOUT_NAME).resolve() / "dist")
+    for package in py_multipackage:
+        run(
+            f"pipx run build . -o {dist_dir}",
+            cwd=Path(util.CHECKOUT_NAME) / package["rel_path"],
         )
-    ]
-    sha = run("git rev-parse HEAD", cwd=util.CHECKOUT_NAME)
-    tags = [dict(ref=f"refs/tags/{tag_name}", object=dict(sha=sha))]
-    open_mock.side_effect = [
-        MockHTTPResponse(releases),
-        MockHTTPResponse(tags),
-        MockHTTPResponse(dict(html_url=url)),
-    ]
+        files.extend(glob(dist_dir + "/*.*"))
 
-    runner(["extract-release", HTML_URL])
-    assert len(open_mock.mock_calls) == 2
-    assert len(get_mock.mock_calls) == len(dist_names) == 3
+    # Finalize the release
+    runner(["tag-release"])
+
+    # Create the release.
+    dist_dir = os.path.join(util.CHECKOUT_NAME, "dist")
+    release = create_draft_release("bar", glob(f"{dist_dir}/*.*"))
+    shutil.rmtree(dist_dir)
+
+    os.environ["RH_RELEASE_URL"] = release.html_url
+    runner(["extract-release"])
 
     log = get_log()
     assert "before-extract-release" not in log
@@ -638,25 +540,46 @@ def test_extract_dist_npm(npm_dist, runner, mocker, open_mock, tmp_path):
 
 
 @pytest.mark.skipif(
-    os.name == "nt", reason="pypiserver does not start properly on Windows"
+    os.name == "nt" and sys.version_info.major == 3 and sys.version_info.minor < 8,
+    reason="See https://bugs.python.org/issue26660",
 )
-def test_publish_assets_py(py_package, runner, mocker, git_prep):
+def test_extract_dist_npm(npm_dist, runner, mocker, mock_github, tmp_path):
+    # Create the release.
+    dist_dir = os.path.join(util.CHECKOUT_NAME, "dist")
+    release = create_draft_release("bar", glob(f"{dist_dir}/*.*"))
+    shutil.rmtree(dist_dir)
+
+    os.environ["RH_RELEASE_URL"] = release.html_url
+    runner(["extract-release"])
+
+    log = get_log()
+    assert "before-extract-release" not in log
+    assert "after-extract-release" in log
+
+
+@pytest.mark.skipif(os.name == "nt", reason="pypiserver does not start properly on Windows")
+def test_publish_assets_py(py_package, runner, mocker, git_prep, mock_github):
     # Create the dist files
     changelog_entry = mock_changelog_entry(py_package, runner, mocker)
-    run("python -m build .", cwd=util.CHECKOUT_NAME)
+    run("pipx run build .", cwd=util.CHECKOUT_NAME)
 
     orig_run = util.run
     called = 0
 
+    os.environ["PYPI_TOKEN_MAP"] = "foo/bar,foo-token\nfizz/buzz,bar"
+
     def wrapped(cmd, **kwargs):
         nonlocal called
-        if cmd.startswith("twine upload"):
-            called += 1
+        if "twine upload" in cmd:
+            if kwargs["env"]["TWINE_PASSWORD"] == "foo-token":
+                called += 1
         return orig_run(cmd, **kwargs)
 
     mock_run = mocker.patch("jupyter_releaser.util.run", wraps=wrapped)
 
     dist_dir = py_package / util.CHECKOUT_NAME / "dist"
+    release = create_draft_release(files=glob(f"{dist_dir}/*.*"))
+    os.environ["RH_RELEASE_URL"] = release.html_url
     runner(["publish-assets", "--dist-dir", dist_dir, "--dry-run"])
     assert called == 2, called
 
@@ -678,20 +601,13 @@ def test_publish_assets_npm(npm_dist, runner, mocker):
 
     mock_run = mocker.patch("jupyter_releaser.util.run", wraps=wrapped)
 
-    runner(
-        [
-            "publish-assets",
-            "--npm-cmd",
-            "npm publish --dry-run",
-            "--dist-dir",
-            dist_dir,
-        ]
-    )
+    runner(["publish-assets", "--npm-cmd", "npm publish --dry-run", "--dist-dir", dist_dir])
 
     assert called == 3, called
 
 
-def test_publish_assets_npm_exists(npm_dist, runner, mocker):
+def test_publish_assets_npm_exists(npm_dist, runner, mocker, mock_github, draft_release):
+    os.environ["RH_RELEASE_URL"] = draft_release
     dist_dir = npm_dist / util.CHECKOUT_NAME / "dist"
     called = 0
 
@@ -705,7 +621,6 @@ def test_publish_assets_npm_exists(npm_dist, runner, mocker):
                 raise err
 
     mock_run = mocker.patch("jupyter_releaser.util.run", wraps=wrapped)
-
     runner(
         [
             "publish-assets",
@@ -721,7 +636,8 @@ def test_publish_assets_npm_exists(npm_dist, runner, mocker):
     assert called == 3, called
 
 
-def test_publish_assets_npm_all_exists(npm_dist, runner, mocker):
+def test_publish_assets_npm_all_exists(npm_dist, runner, mocker, mock_github, draft_release):
+    os.environ["RH_RELEASE_URL"] = draft_release
     dist_dir = npm_dist / util.CHECKOUT_NAME / "dist"
     called = 0
 
@@ -734,34 +650,29 @@ def test_publish_assets_npm_all_exists(npm_dist, runner, mocker):
             raise err
 
     mocker.patch("jupyter_releaser.util.run", wraps=wrapped)
-
-    with pytest.raises(ValueError):
-        runner(
-            [
-                "publish-assets",
-                "--npm-token",
-                "abc",
-                "--npm-cmd",
-                "npm publish --dry-run",
-                "--dist-dir",
-                dist_dir,
-            ]
-        )
+    runner(
+        [
+            "publish-assets",
+            "--npm-token",
+            "abc",
+            "--npm-cmd",
+            "npm publish --dry-run",
+            "--dist-dir",
+            dist_dir,
+        ]
+    )
 
     assert called == 3, called
 
 
-def test_publish_release(npm_dist, runner, mocker, open_mock):
-    open_mock.side_effect = [MockHTTPResponse([REPO_DATA]), MockHTTPResponse()]
-    dist_dir = npm_dist / util.CHECKOUT_NAME / "dist"
-    runner(["publish-release", HTML_URL])
+def test_publish_release(npm_dist, runner, mocker, mock_github, draft_release):
+    os.environ["RH_RELEASE_URL"] = draft_release
+    runner(["publish-release"])
 
     log = get_log()
     assert "before-publish-release" in log
     assert "after-publish-release" in log
 
-    assert len(open_mock.call_args) == 2
-
 
 def test_config_file(py_package, runner, mocker, git_prep):
 
@@ -775,7 +686,7 @@ def test_config_file(py_package, runner, mocker, git_prep):
 
     def wrapped(cmd, **kwargs):
         nonlocal called
-        if cmd.startswith("python -m build --outdir foo"):
+        if cmd.startswith("pipx run build --outdir foo"):
             called = True
             return ""
         return orig_run(cmd, **kwargs)
@@ -797,7 +708,7 @@ def test_config_file_env_override(py_package, runner, mocker, git_prep):
 
     def wrapped(cmd, **kwargs):
         nonlocal called
-        if cmd.startswith("python -m build --outdir bar"):
+        if cmd.startswith("pipx run build --outdir bar"):
             called = True
             return ""
         return orig_run(cmd, **kwargs)
@@ -819,7 +730,7 @@ def test_config_file_cli_override(py_package, runner, mocker, git_prep):
 
     def wrapped(cmd, **kwargs):
         nonlocal called
-        if cmd.startswith("python -m build --outdir bar"):
+        if cmd.startswith("pipx run build --outdir bar"):
             called = True
             return ""
         return orig_run(cmd, **kwargs)
@@ -834,9 +745,9 @@ def test_config_file_cli_override(py_package, runner, mocker, git_prep):
     assert "after-build-python" in log
 
 
-def test_forwardport_changelog_no_new(npm_package, runner, mocker, open_mock, git_prep):
-
-    open_mock.side_effect = [MockHTTPResponse([REPO_DATA]), MockHTTPResponse()]
+def test_forwardport_changelog_no_new(npm_package, runner, mocker, mock_github, git_prep):
+    release = create_draft_release("bar")
+    os.environ["RH_RELEASE_URL"] = release.html_url
 
     # Create a branch with a changelog entry
     util.run("git checkout -b backport_branch", cwd=util.CHECKOUT_NAME)
@@ -846,30 +757,25 @@ def test_forwardport_changelog_no_new(npm_package, runner, mocker, open_mock, gi
     util.run(f"git tag v{VERSION_SPEC}", cwd=util.CHECKOUT_NAME)
 
     # Run the forwardport workflow against default branch
-    runner(["forwardport-changelog", HTML_URL])
-
-    assert len(open_mock.mock_calls) == 2
+    runner(["forwardport-changelog"])
 
     log = get_log()
     assert "before-forwardport-changelog" in log
     assert "after-forwardport-changelog" in log
 
 
-def test_forwardport_changelog_has_new(
-    npm_package, runner, mocker, open_mock, git_prep
-):
+def test_forwardport_changelog_has_new(npm_package, runner, mocker, mock_github, git_prep):
+    release = create_draft_release("bar")
+    os.environ["RH_RELEASE_URL"] = release.html_url
 
-    open_mock.side_effect = [MockHTTPResponse([REPO_DATA]), MockHTTPResponse()]
-    current = util.run("git branch --show-current")
+    current = util.run("git branch --show-current", cwd=util.CHECKOUT_NAME)
 
     # Create a branch with a changelog entry
     util.run("git checkout -b backport_branch", cwd=util.CHECKOUT_NAME)
     util.run("git push origin backport_branch", cwd=util.CHECKOUT_NAME)
     util.run(f"git checkout {current}")
     mock_changelog_entry(npm_package, runner, mocker)
-    util.run(
-        f'git commit -a -m "Add changelog entry {VERSION_SPEC}"', cwd=util.CHECKOUT_NAME
-    )
+    util.run(f'git commit -a -m "Add changelog entry {VERSION_SPEC}"', cwd=util.CHECKOUT_NAME)
     util.run(f"git tag v{VERSION_SPEC}", cwd=util.CHECKOUT_NAME)
     util.run(f"git checkout {current}", cwd=util.CHECKOUT_NAME)
     util.run("git push origin backport_branch --tags", cwd=util.CHECKOUT_NAME)
@@ -886,9 +792,8 @@ def test_forwardport_changelog_has_new(
     # Run the forwardport workflow against default branch
     url = osp.abspath(npm_package)
     os.chdir(npm_package)
-    runner(["forwardport-changelog", HTML_URL, "--branch", current])
+    runner(["forwardport-changelog", "--branch", current])
 
-    assert len(open_mock.call_args) == 2
     util.run(f"git checkout {current}", cwd=npm_package)
 
     expected = """
@@ -905,3 +810,13 @@ def test_forwardport_changelog_has_new(
 ## 1.0.1
 """
     assert expected in text, text
+
+
+def test_ensure_sha(npm_package, runner, git_prep):
+    sha = util.run("git rev-parse HEAD", cwd=util.CHECKOUT_NAME)
+    current = util.run("git branch --show-current", cwd=util.CHECKOUT_NAME)
+    runner(["ensure-sha", "--branch", current, "--expected-sha", sha])
+    runner(["ensure-sha", "--branch", current, "--expected-sha", "abc", "--dry-run"])
+
+    with pytest.raises(RuntimeError):
+        runner(["ensure-sha", "--branch", current, "--expected-sha", "abc"])
